#
# Copyright (c) 2008-2015 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

__docformat__ = 'restructuredtext'


# import standard library
import inspect
import json
import os
import venusian

# import interfaces
from pyams_form.interfaces.form import IFormLayer, IResetWidget, ICloseWidget, IDateWidget, IDatetimeWidget, ITimeWidget, \
    IColorWidget, IHTMLWidget
from pyams_form.schema import IResetButton, ICloseButton
from pyams_skin.interfaces.tinymce import ITinyMCEConfiguration
from pyams_utils.schema import IColorField, IHTMLField
from pyramid.interfaces import IRequest
from z3c.form.interfaces import INPUT_MODE, IFieldWidget, IButtonAction, IWidgetLayoutTemplate, IDataConverter
from zope.pagetemplate.interfaces import IPageTemplate
from zope.schema.interfaces import IDate, IDatetime, ITime

# import packages
from pyams_utils.adapter import adapter_config
from pyramid.exceptions import ConfigurationError
from z3c.form.action import Action
from z3c.form.browser.submit import SubmitWidget
from z3c.form.browser.text import TextWidget
from z3c.form.browser.textarea import TextAreaWidget
from z3c.form.button import ButtonAction
from z3c.form.converter import BaseDataConverter
from z3c.form.widget import FieldWidget, WidgetTemplateFactory, WidgetLayoutFactory
from zope.interface import implementer_only, directlyProvides, Interface


#
# Widget template configuration
#

class widgettemplate_config(object):
    """Class decorator used to declare a widget template"""

    venusian = venusian  # for testing injection

    def __init__(self, **settings):
        if 'for_' in settings:
            if settings.get('context') is None:
                settings['context'] = settings['for_']
        self.__dict__.update(settings)

    def __call__(self, wrapped):
        settings = self.__dict__.copy()

        def callback(context, name, ob):
            template = os.path.join(os.path.dirname(inspect.getfile(inspect.getmodule(ob))),
                                    settings.get('template'))
            if not os.path.isfile(template):
                raise ConfigurationError("No such file", template)

            contentType = settings.get('contentType', 'text/html')
            factory = WidgetTemplateFactory(template, contentType)
            provides = settings.get('provides', IPageTemplate)
            directlyProvides(factory, provides)

            config = context.config.with_package(info.module)
            config.registry.registerAdapter(factory,
                                            (settings.get('context', Interface),
                                             settings.get('layer', IRequest),
                                             settings.get('view', None),
                                             settings.get('field', None),
                                             settings.get('widget', ob)),
                                            provides,
                                            settings.get('mode', INPUT_MODE))

        info = self.venusian.attach(wrapped, callback, category='pyams_form')

        if info.scope == 'class':
            # if the decorator was attached to a method in a class, or
            # otherwise executed at class scope, we need to set an
            # 'attr' into the settings if one isn't already in there
            if settings.get('attr') is None:
                settings['attr'] = wrapped.__name__

        settings['_info'] = info.codeinfo  # fbo "action_method"
        return wrapped


class widgetlayout_config(object):
    """Class decorator used to declare a widget layout"""

    venusian = venusian  # for testing injection

    def __init__(self, **settings):
        if 'for_' in settings:
            if settings.get('context') is None:
                settings['context'] = settings['for_']
        self.__dict__.update(settings)

    def __call__(self, wrapped):
        settings = self.__dict__.copy()

        def callback(context, name, ob):
            template = os.path.join(os.path.dirname(inspect.getfile(inspect.getmodule(ob))),
                                    settings.get('template'))
            if not os.path.isfile(template):
                raise ConfigurationError("No such file", template)

            contentType = settings.get('contentType', 'text/html')
            factory = WidgetLayoutFactory(template, contentType)
            provides = settings.get('provides', IWidgetLayoutTemplate)
            directlyProvides(factory, provides)

            config = context.config.with_package(info.module)
            config.registry.registerAdapter(factory,
                                            (settings.get('context', Interface),
                                             settings.get('layer', IRequest),
                                             settings.get('view', None),
                                             settings.get('field', None),
                                             settings.get('widget', ob)),
                                            provides,
                                            settings.get('mode', INPUT_MODE))

        info = self.venusian.attach(wrapped, callback, category='pyams_form')

        if info.scope == 'class':
            # if the decorator was attached to a method in a class, or
            # otherwise executed at class scope, we need to set an
            # 'attr' into the settings if one isn't already in there
            if settings.get('attr') is None:
                settings['attr'] = wrapped.__name__

        settings['_info'] = info.codeinfo  # fbo "action_method"
        return wrapped


#
# Reset button widget and action
#

@widgettemplate_config(mode='input', template='templates/reset-input.pt', layer=IFormLayer)
@widgettemplate_config(mode='display', template='templates/reset-display.pt', layer=IFormLayer)
@implementer_only(IResetWidget)
class ResetWidget(SubmitWidget):
    """A reset button of a form."""

    klass = u'reset-widget'
    css = u'reset'


@adapter_config(context=(IResetButton, IFormLayer), provides=IFieldWidget)
def ResetFieldWidget(field, request):
    reset = FieldWidget(field, ResetWidget(request))
    reset.value = field.title
    return reset


@adapter_config(context=(IFormLayer, IResetButton), provides=IButtonAction)
class ResetButtonAction(ResetWidget, ButtonAction):
    """Reset button action"""

    def __init__(self, request, field):
        Action.__init__(self, request, field.title)
        ResetWidget.__init__(self, request)
        self.field = field


#
# Close button widget and action
#

@widgettemplate_config(mode='input', template='templates/close-input.pt', layer=IFormLayer)
@widgettemplate_config(mode='display', template='templates/close-display.pt', layer=IFormLayer)
@implementer_only(ICloseWidget)
class CloseWidget(SubmitWidget):
    """A dialog close button"""

    klass = u'close-widget'
    css = u'close'


@adapter_config(context=(ICloseButton, IFormLayer), provides=IFieldWidget)
def CloseFieldWidget(field, request):
    close = FieldWidget(field, CloseWidget(request))
    close.value = field.title
    return close


@adapter_config(context=(IFormLayer, ICloseButton), provides=IButtonAction)
class CloseButtonAction(CloseWidget, ButtonAction):
    """Close button action"""

    def __init__(self, request, field):
        Action.__init__(self, request, field.title)
        CloseWidget.__init__(self, request)
        self.field = field


#
# Date widget
#

@widgettemplate_config(mode='input', template='templates/date-input.pt', layer=IFormLayer)
@implementer_only(IDateWidget)
class DateWidget(TextWidget):
    """Date widget"""


@adapter_config(context=(IDate, IFormLayer), provides=IFieldWidget)
def DateFieldWidget(field, request):
    return FieldWidget(field, DateWidget(request))


#
# Datetime widget
#

@widgettemplate_config(mode='input', template='templates/datetime-input.pt', layer=IFormLayer)
@implementer_only(IDatetimeWidget)
class DatetimeWidget(TextWidget):
    """Datetime widget"""


@adapter_config(context=(IDatetime, IFormLayer), provides=IFieldWidget)
def DatetimeFieldWidget(field, request):
    return FieldWidget(field, DatetimeWidget(request))


#
# Time widget
#

@widgettemplate_config(mode='input', template='templates/time-input.pt', layer=IFormLayer)
@implementer_only(ITimeWidget)
class TimeWidget(TextWidget):
    """Time widget"""


@adapter_config(context=(ITime, IFormLayer), provides=IFieldWidget)
def TimeFieldWidget(field, request):
    return FieldWidget(field, TimeWidget(request))


#
# Color widget
#

@adapter_config(context=(IColorField, IColorWidget), provides=IDataConverter)
class ColorDataConverter(BaseDataConverter):
    """Color field data converter"""

    def toWidgetValue(self, value):
        value = super(ColorDataConverter, self).toWidgetValue(value)
        if value:
            value = '#' + value
        return value

    def toFieldValue(self, value):
        if value and value.startswith('#'):
            value = value[1:]
        return super(ColorDataConverter, self).toFieldValue(value)


@widgettemplate_config(mode='input', template='templates/color-input.pt', layer=IFormLayer)
@implementer_only(IColorWidget)
class ColorWidget(TextWidget):
    """Color widget"""


@adapter_config(context=(IColorField, IFormLayer), provides=IFieldWidget)
def ColorFieldWidget(field, request):
    return FieldWidget(field, ColorWidget(request))


#
# HTML widget
#

@widgettemplate_config(mode='input', template='templates/html-input.pt', layer=IFormLayer)
@implementer_only(IHTMLWidget)
class HTMLWidget(TextAreaWidget):
    """HTML editor widget"""

    label_css_class = 'textarea'

    @property
    def editor_data(self):
        registry = self.request.registry
        config = registry.queryMultiAdapter((self.form, self.request), ITinyMCEConfiguration, name=self.name)
        if config is None:
            config = registry.queryMultiAdapter((self.form, self.request), ITinyMCEConfiguration)
        if config is None:
            config = getattr(self, 'editor_config_data', None)
            if config is not None:
                return json.dumps(config)
        else:
            return json.dumps(config.configuration)


@adapter_config(context=(IHTMLField, IFormLayer), provides=IFieldWidget)
def HTMLFieldWidget(field, request):
    return FieldWidget(field, HTMLWidget(request))

